--********************************************************************************
--* sumFilesForUser.rpt.ms v1.1                                                  *
--* By Ofer Zelichover (c) 02/2002                                               *
--* Last update: 2012-01-21                                                      *
--*                                                                              *
--* This is a report type definition file, used by timeReporter.                 *
--*                                                                              *
--* This file comes with no warranty. For detailed warranty description and terms*
--* of use, please see the readme.txt file in the <max_root>\timeTracker folder. *
--*                                                                              *
--********************************************************************************
--*    CHANGE AT YOUR OWN RISK !!!                                               *
--********************************************************************************

(
	local report = report_struct name:"User - Summed Time Per File" \
		desc:"Lists the summed up time spent on each file by the selected user." \
		params:#(
			time_struct hoursPerDay:ttSettings.hoursPerWorkDay, -- total time
			0, -- username
			#(2000, 1, 1), -- start date
			#(2020, 12, 31), -- end date
			#(), -- file list
			#() -- file total time
		) \ -- end of params

		startFn:(fn startFunc pList cList uList &params = (
				params[1] = params[1].strToTime "0:0:0:0" -- reset total time counter
				params[5] = #()
				params[6] = #()
				local str = "" as stringStream
				format "User: % - %\n" (uList[params[2]].username) (uList[params[2]].realname) usr to:str
				format "----------------------------------------------------------------\n" to:str
				str as string
			)--end startFunc
		) \ -- end startFn

		lineFn:(fn lineFunc pList cList uList sess &params = (
				if sess.user == uList[params[2]].userName AND \
				  (rtGenFuncs.compareDates sess.startDate params[3] mode:">=") AND \
				  (rtGenFuncs.compareDates sess.endDate params[4] mode:"<=") then (

					-- add session time to total time
					local n = findItem params[5] sess.file
					if n == 0 then (
						append params[5] sess.file
						append params[6] (params[1].add sess.sessionTime)
					) else (
						params[6][n] = params[6][n].add sess.sessionTime
					)

					-- generate output string
					""
				)
			)--end lineFunc
		) \ -- end lineFn

		endFn:(fn endFunc pList cList uList &params = (
				for i in params[6] do (params[1] = params[1].add i)

				local str = "" as stringStream
				local w = amax (for i in params[5] collect i.count)
				for i = 1 to params[5].count do (
					format "% - %\n" (rtGenFuncs.fixedWidth params[5][i] w) (params[6][i].timeToStr()) to:str
				)
				format "================================================================\n" to:str
				format "Total Time: %\n" (params[1].timeToStr()) to:str
				str as string
			)-- end endFunc
		) \ --end endFn

		interface:(
			rollout ReportUI "Report Parameters"
			(
				group "User: " (
					dropdownlist repUserList "" items:(ro_timeReporter.getUsers())
				)
				group "Date Ranges: " (
					spinner repFromMonth "From:     Month:" fieldWidth:30 range:[1,12,1] type:#integer across:3 offset:[15,0]
					spinner repFromDay "Day:" fieldWidth:30 range:[1,31,1] type:#integer offset:[5,0]
					spinner repFromYear "Year:" fieldWidth:40 range:[2000,2020,2000] type:#integer

					spinner repToMonth "To:         Month:" fieldWidth:30 range:[1,12,12] type:#integer across:3 offset:[15,0]
					spinner repToDay "Day:" fieldWidth:30 range:[1,31,31] type:#integer offset:[5,0]
					spinner repToYear "Year:" fieldWidth:40 range:[2000,2020,2020] type:#integer
				)
				
				fn changeDateDay date d = (date[3] = d; date)
				fn changeDateMonth date m = (date[2] = m; date)
				fn changeDateYear date y = (date[1] = y; date)
				
				
				on repFromMonth changed val do (rtGenFuncs.setParameter 3 (changeDateMonth (rtGenFuncs.getParameter 3) val))
				on repFromDay changed val do (rtGenFuncs.setParameter 3 (changeDateDay (rtGenFuncs.getParameter 3) val))
				on repFromYear changed val do (rtGenFuncs.setParameter 3 (changeDateYear (rtGenFuncs.getParameter 3) val))
				
				on repToMonth changed val do (rtGenFuncs.setParameter 4 (changeDateMonth (rtGenFuncs.getParameter 4) val))
				on repToDay changed val do (rtGenFuncs.setParameter 4 (changeDateDay (rtGenFuncs.getParameter 4) val))
				on repToYear changed val do (rtGenFuncs.setParameter 4 (changeDateYear (rtGenFuncs.getParameter 4) val))

				on repUserList selected item do (rtGenFuncs.setParameter 2 item)
				on ReportUI open do (rtGenFuncs.setParameter 2 repUserList.selection)
				
			)
		)-- end interface definition


	report
)