--******************************************************************************
--*  Time Reporter v1.2                 for Max 4.2                            *
--*  (A "Time Tracker" Tool)                                                   *
--*  by Ofer Zelichover (c) 01/2002                                            *
--*  www.oferz.com  ;   maxscripts@oferz.com                                   *
--*                                                                            *
--******************************************************************************
--*  You may use this script freely as you see fit.                            *
--*  You may use parts or the script as a whole in your own scripts.           *
--*  (it would be nice if you give me a credit if you do so ;))                *
--******************************************************************************
--*  This script comes with no waranty!                                        *
--*  Although I tried this script and couldn't find any problems with it, I can*
--*  in no way be held responsible for any kind of loss or damage, whether     *
--*  direct or indirect, due to the use of this script.                        *
--*                                                                            *
--*  ********************************************************************      *
--*  *** IF YOU DON'T LIKE THE ABOVE STATEMENT, DON'T USE THIS SCRIPT ***      *
--*  ********************************************************************      *
--*                                                                            *
--*  **** This script was written for max 4.2 and wasn't tested on other ****  *
--*  **** versions of max.                                               ****  *
--*                                                                            *
--*  If you find any bugs in this script, please let me know.                  *
--******************************************************************************
--* Description                                                                *
--* -------------                                                              *
--*  Generate simple reports based on the data saved with "Time Tracker"       *
--*                                                                            *
--******************************************************************************
--* History, Status and Known issues                                           *
--* ----------------------------------                                         *
--* Created: 27/02/2002 - Ver 1.0                                              *
--*                                                                            *
--* Known issues:                                                              *
--*                                                                            *
--* Need to add:                                                               *
--*                                                                            *
--******************************************************************************
--* Isntallation:                                                              *
--* --------------                                                             *
--*    requires:                                                               *
--*         "Time Tracker" must be installed to use this script.               *
--*                                                                            *
--*    put:                                                                    *
--*         timeReporter-v1.ms     - anywhere                                  *
--*                                                                            *
--*  After you run the script, the Time Reporter tool will apear in the        *
--*  "Time Traker" category in the customize ui menu.                          *
--*                                                                            *
--******************************************************************************


macroScript timeReporter
	category:"Time Tracker"
	buttonText:"timeReporter"
	tooltip:"Time Reporter 1.2"
	icon:#("timeReporter", 1)
(

-- Global Variable Declerations
-----------------------------------
global ro_timeReporter
global timeReporterFloater
global trReportList
global s_Date
global rtGenFuncs
global report_struct
global rtGenFuncs_struct

-- Global Struct Definitions
----------------------------------
	struct report_struct (name, desc, 
		params,
		startFn,
		lineFn,
		endFn,
		interface
	)-- end report_struct


	struct rtGenFuncs_struct (
		-- Initialize a date struct instance.
		dateStruct = s_Date(),
		fn compareDates d1 d2 mode:"==" dateFormat: = (
			try (
				-- Set the date format if needed
				if isKindOf dateFormat string then
					dateStruct.format = dateFormat
				-- Convert the input dates to 
				d1 = dateStruct.dateToInt d1
				d2 = dateStruct.dateToInt d2
				case mode of (
					"==":	d1 == d2
					"!=":	d1 != d2
					">":	d1 > d2
					"<":	d1 < d2
					"<=":	d1 <= d2
					">=":	d1 >= d2
				)
			)catch()
		),
		fn setParameter i val = (try (trReportList[ro_timeReporter.trReportsDisplayList.selection].params[i] = val)catch()),
		fn getParameter i = (try (trReportList[ro_timeReporter.trReportsDisplayList.selection].params[i])catch()),
		fn fixedWidth str w = (
			if str.count < w then while (str.count<w) do str+=" "
			if str.count > w then str = (subString str 1 (w-3)) + "..."
			str
		)
	) --end of rtGenFuncs_struct 

-- Rollout Definition
--------------------------
	rollout ro_timeReporter "Select Report"
	(
	-- Local Struct Declerations
	------------------------------
		struct trSession_struct (startTime, startDate, endTime, endDate, sessionTime, user, projectID, file)
	
		struct proj_struct (ID=0, name="", dueTime="", clientID=0, comments="", 
			totalTime=(time_struct hoursPerDay:ttSettings.hoursPerWorkDay)
		)--end proj_struct
	
		struct client_struct (ID, name, address, phone, cellphone, fax, email)
	
		struct user_struct (username, realname)
	
	
	-- Local Variable Declerations
	--------------------------------
		local prevReport = 0
		local trProjList = #()
		local trClientList = #()
		local trUserList = #()
	
	-- GUI
	--------
		group "Report Type: " (
			dropdownlist trReportsDisplayList ""
			label trReportDescLbl "Description: " align:#left
			edittext trReportDesc "" height:60
		)
	
		group "Output: " (
			label trFilenameLbl "Save To File:" align:#left
			edittext trFilename "" across:2 align:#left width:250 offset:[-5,0]
			button trFilenameBrowse "Browse..." height:17 width:55 align:#right offset:[5,0]
			checkbox trViewOutput "View Report." checked:true
		)
	
		group "" (
			label trStatus "Status: Ready." align:#left
			progressbar trProgBar "" 
		)
	
		button trGo "Execute Report" width:130 height:25 offset:[7,0] across:3
		button trAbout "About" width:80 offset:[30,2]
		button trClose "Close" width:80 offset:[19,2]
	
	
	-- Functions
	--------------
	-- General Functions
		fn getFileLength fName = (
			local l = 0
			local f = fOpen fName "rb"
			try (
				fSeek f 0 #seek_end
				l = fTell f
			)catch(l = 0)
			fClose f
			l
		)
	
		fn getReportNames = (
			for i in trReportList collect i.name
		)

		fn sortReportList &list = (
			local compareFn = (fn compareVals v1 v2 = (
					case of (
						(v1.name>v2.name): 1
						(v1.name==v2.name): 0
						(v1.name<v2.name): -1
					)
				)
			)
			qsort list compareFn
		)

		fn trimSpaces str =
		(
			trimLeft (trimRight str)
		)
		
		fn strToSession str = (
			local s = trSession_struct()
			local vals = filterString str ";"
			s.startTime = try((time_struct()).strToTime (trimSpaces vals[1]))catch(time_struct())
			s.startDate = try(trimSpaces vals[2])catch("")
			s.endTime = try((time_struct()).strToTime (trimSpaces vals[3]))catch(time_struct())
			s.endDate = try(trimSpaces vals[4])catch("")
			s.sessionTime = try((time_struct hoursPerDay:ttSettings.hoursPerWorkDay).strToTime (trimSpaces vals[5]))catch(time_struct hoursPerDay:ttSettings.hoursPerWorkDay)
			s.user = try(trimSpaces vals[6])catch("")
			s.projectID = try((trimSpaces vals[7]) as integer)catch(0)
			s.file = try(trimSpaces vals[8])catch("")
			s
		)
	
		fn filterValue s = (
			local n = findString s ">="
			if n == undefined then s
			else subString s (n+2) (s.count-(n+1))
		)
		
		fn replaceChars str rep repwith = (
			local n
			while ((n = findString str rep) != undefined) do (
				str = replace str n rep.count repwith
			)
			str
		)
	
		fn isAllowedLetter s = (
			local letters = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"
			local numbers = "0123456789"
			local signs = ".,\"`':!$%&~()[]{}<>?/\\"
			findString letters s != undefined OR \
			findString numbers s != undefined OR \
			findString signs s != undefined
		)
	
		fn breakDescLines str w = (
			local s = ""
			local l = ""
			local word = ""
			for i = 1 to str.count do (
				if isAllowedLetter str[i] then (
					word += str[i]
				) else (
					l += word + str[i]
					word = ""
				)
				if (getTextExtent (l+word)).x >= w OR str[i]=="\n" then (
					l += "\r\n"
					s += l
					l = ""
				)
			)
			s + l + word
		)
	
	
	-- settings functions
			fn checkSettings = (
				if ttSettings.hoursPerWorkDay < 1 OR ttSettings.hoursPerWorkDay > 24 then
					ttSettings.hoursPerWorkDay = 8
				if ttSettings.tickInterval < (30*1000) OR ttSettings.tickInterval > (3600*1000) then
					ttSettings.tickInterval = (60*1000)
				if classOf ttSettings.askBeforeQuit != BooleanClass then
					ttSettings.askBeforeQuit = true
			)
	
			fn loadSettings = (
				local fName = ttSettings.baseDir + "timeTracker.ini"
				local str = (getINISetting fName "Settings" "hoursPerWorkDay") as integer
				if str != undefined then ttSettings.hoursPerWorkDay = str
				str = (getINISetting fName "Settings" "tickInterval") as integer
				if str != undefined then ttSettings.tickInterval = str
				str = (getINISetting fName "Settings" "askBeforeQuit")=="true"
				if str != undefined then ttSettings.askBeforeQuit = str
				try (
					local arr = filterString (getINISetting fName "Settings" "dialogPos") "[,]"
					ttSettings.dialogPos = [(arr[1] as integer), (arr[2] as integer)]
				)catch()
				try (
					local arr = filterString (getINISetting fName "Settings" "smallDialogPos") "[,]"
					ttSettings.smallDialogPos = [(arr[1] as integer), (arr[2] as integer)]
				)catch()
				checkSettings()
			)
	
	-- Users Functions
		fn loadUserList = (
			local fName = ttSettings.dataDir + "users.dat"
			local f = openFile fName mode:"r"
			if f != undefined then (
				trUserList = #()
				try (
					while NOT (eof f) do (
						local a = filterString (readLine f) "="
						if a.count==2 then (
							local newUser = user_struct username:a[1] realname:a[2]
							append trUserList newUser
						)
					)
				)catch()
				close f
			)
		)-- end loadUserList
	
		fn getUsers = (for i in trUserList collect (i.userName + " - " + i.realName))
		
		fn getUserNumInList usr = try(findItem (for i in trUserList collect i.username) usr)catch(0)
	
	-- Clients Functions
		fn loadClientList = (
			local returnVal = true
			local fName = ttSettings.dataDir + "clients.dat"
			local f = openFile fName mode:"r"
			if f != undefined then (
				trClientList = #()
				local newClient
				try (
					while (NOT (eof f)) do (
						local l = readLine f
						if findString l "[Client_" != undefined then (
							if newClient != undefined then append trClientList newClient
							newClient = client_struct()
						)
						if newClient != undefined then (
							if (findString l "<ID>=") == 1 then newClient.ID = (filterValue l) as integer
							if (findString l "<name>=") == 1 then newClient.name = filterValue l
							if (findString l "<address>=") == 1 then newClient.address = (replaceChars (replaceChars (filterValue l) "<&R>" "\r") "<&N>" "\n")
							if (findString l "<phone>=") == 1 then newClient.phone = filterValue l
							if (findString l "<cellPhone>=") == 1 then newClient.cellPhone = filterValue l
							if (findString l "<fax>=") == 1 then newClient.fax = filterValue l
							if (findString l "<email>=") == 1 then newClient.email = filterValue l
						)
					)
				)catch(print "Error loading client list.";returnVal = false)
				if newClient != undefined then append trClientList newClient
				close f
			) else (returnVal = false)
			returnVal
		)-- end loadClientList
	
		fn getClients = (for i in trClientList collect i.name)
		
		fn getClientIDInList i = try((for i in trClientList collect i.ID)[i])catch(0)
	
	-- Projects Functions
		fn loadProjectList = (
			local returnVal = true
			local fName = ttSettings.dataDir + "projects.dat"
			local f = openFile fName mode:"r"
			if f != undefined then (
				trProjList = #()
				local newProj
				try (
					while (NOT (eof f)) do (
						local l = readLine f
						if findString l "[Project_" != undefined then (
							if newProj != undefined then append trProjList newProj
							newProj = proj_struct()
						)
						if newProj != undefined then (
							if (findString l "<ID>=") == 1 then newProj.ID = (filterValue l) as integer
							if (findString l "<name>=") == 1 then newProj.name = filterValue l
							if (findString l "<dueTime>=") == 1 then newProj.dueTime = filterValue l
							if (findString l "<clientID>=") == 1 then newProj.clientID = (filterValue l) as integer
							if (findString l "<totalTime>=") == 1 then
								newProj.totalTime = (time_struct hoursPerDay:ttSettings.hoursPerWorkDay).strToTime (filterValue l)
							if (findString l "<comments>=") == 1 then newProj.comments = (replaceChars (replaceChars (filterValue l) "<&R>" "\r") "<&N>" "\n")
						)
					)
				)catch(print "Error loading project list.";returnVal = false)
				if newProj != undefined then append trProjList newProj
				close f
			) else (returnVal = false)
			returnVal
		)-- end loadProjectList
	
		fn getProjects = (for i in trProjList collect i.name)
		
		fn getProjIDInList i = try((for i in trProjList collect i.ID)[i])catch(0)
	
	-- GUI Functions
		fn updateGUI = (
			local i = trReportsDisplayList.selection
			trReportDesc.text = breakDescLines (try(trReportList[i].desc)catch("")) 285
		)
	
		fn changeReportType i = (
			if i == 0 then return()
			try(removeRollout trReportList[prevReport].interface timeReporterFloater)catch()
			try(addRollout trReportList[i].interface timeReporterFloater)catch()
			updateGUI()
			prevReport = i
		)
	
		fn execReport = (
			local outFilename = trFilename.text
			if outFilename == "" then 
				outFilename = sysInfo.tempDir + "timeReport.txt"
			trStatus.text = "Status: Executeing report."
			trProgBar.value = 0 
			local fName = ttSettings.dataDir + "Sessions.dat"
			-- get file length
			local fileLength = getFileLength fName

			-- open the sessions file
			local sessFile = openFile fName
			
			-- check if sessions file was opened ok.
			if sessFile == undefined then (
				trStatus.text = "Status: Error - could not open sessions file."
				return()
			)
			
			local outFile = createFile outFilename
	
			-- check if sessions file was opened ok.
			if outFile == undefined then (
				try(close sessFile)catch()
				trStatus.text = "Status: Error - could not create output file."
				return()
			)
			
			-- execute the report
			try (
				local i = trReportsDisplayList.selection
				local report = trReportList[i]
				local outStr
				outStr = report.startFn trProjList trClientList trUserList &report.params
				if outStr!= undefined then format "%" outStr to:outFile
				while NOT (eof sessFile) do (
					local sess = strToSession (readLine sessFile)
					outStr = report.lineFn trProjList trClientList trUserList sess &report.params
					if outStr!= undefined then format "%" outStr to:outFile
					trProgBar.value = (((filePos sessFile)/fileLength as float))*100.
				)
				outStr = report.endFn trProjList trClientList trUserList &report.params
				if outStr!= undefined then format "%" outStr to:outFile
			)catch()
	
			-- close the files
			close sessFile
			flush outFile
			close outFile
	
			-- update the status line
			trStatus.text = "Status: Ready."
			
			-- display the output file
			if trViewOutput.state then
				edit outFilename
			
			-- clean up
			gc()
		)

		fn aboutDlg = (
			local msg = "
                                           Time Reporter v1.1
                                 by Ofer Zelichover (c) 02/2002
                    http://www.oferz.com   ||   maxscripts@oferz.com

 About:
----------
Time Reporter is a tool to generate simple reports using the data
saved with Time Tracker.


 Warranty:
-----------------
This script comes with no waranty!
Although I tried this script and couldn't find any problems with it, I can
in no way be held responsible for any kind of loss or damage, whether
direct or indirect, due to the use of this script.

************************************************************
                    IF YOU DON'T LIKE THE ABOVE STATEMENT,
                               PLEASE DON'T USE THIS SCRIPT
************************************************************


You may use this script freely as you see fit.
You may use parts or the script as a whole in your own scripts.
(it would be nice if you give me a credit if you do so ;))


This script was written for max 2008 and wasn't tested on other
versions of max.


If you find any bugs in this script, please let me know.
"
			messageBox msg title:"About Time Reporter" beep:false
		)

		fn init = (
			-- create a message dialog 
			local msgDlg
			rollout msgDlg "Time Reporter" (
				label lbl1 "Loading report types, please wait..."
			)
			createDialog msgDlg width:200 height:20 style:#(#style_border)
			
			try (
				-- load settings a databases
				loadSettings()
				checkSettings()
				loadUserList()
				loadClientList()
				loadProjectList()
				
				-- load report types
				trReportList = #()
				local reportDir = ttSettings.baseDir + "reports\\"
				local reportExt = ".rpt.ms"
				for f in (getFiles (reportDir + "*" + reportExt)) do (
					try(
						append trReportList (fileIn f)
					)catch(format "Error reading report definition file:\n --> %\n" f)
				)
				for i = trReportList.count to 1 by -1 where (trReportList[i]==undefined) do
					deleteItem trReportList i
				sortReportList &trReportList
				trReportsDisplayList.items = getReportNames()
				changeReportType trReportsDisplayList.selection
			)catch()
			
			-- close message dialog
			destroyDialog msgDlg
			
			gc()
		)
	
		fn done = (
			trReportList = #()
			gc()
		)
		
	-- Event Handlers
	-------------------
		on trFilenameBrowse pressed do (
			local bf = filenameFromPath trFilename.text
			if bf == "" then bf = "timeReport.txt"
			local f = getSaveFilename caption:"Select report output file..." filename:bf types:"Text Files (*.txt)|*.txt|All Files (*.*)|*.*"
			if f != undefined then
				trFilename.text = f
		)
	
		on trReportsDisplayList selected item do (changeReportType item)
		on trReportDesc changed text do (updateGUI())
	
		on trGo pressed do (execReport())
		on trAbout pressed do (aboutDlg())
		on trClose pressed do (closeRolloutFloater timeReporterFloater)
	
		on ro_timeReporter open do init()
		on ro_timeReporter close do done()
	) -- end of ro_timeReporter rollout
	
	
	on execute do (
		rtGenFuncs = rtGenFuncs_struct()
		trReportList = #()
		try(closeRolloutFloater timeReporterFloater)catch()
		timeReporterFloater = newRolloutFloater "Time Reporter 1.2" 350 530
		addRollout ro_timeReporter timeReporterFloater
	)
	
) -- end of macroscript